package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.MaterialImageInfo;
import cn.com.ctop.common.module.entity.MaterialInfo;
import cn.com.ctop.common.module.mapper.MaterialInfoMapper;
import cn.com.ctop.common.module.mapper.SysRoleMapper;
import cn.com.ctop.common.module.service.IMaterialImageInfoService;
import cn.com.ctop.common.module.service.IMaterialInfoService;
import cn.com.ctop.common.module.utils.Check;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.DateUtils;
import org.jeecg.modules.ctop.service.IProjectMemberService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 素材
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-03-16
 */
@Slf4j
@Api(tags = "素材")
@RestController
@RequestMapping("/ctop/MaterialImageInfo")
public class MaterialImageInfoController {
    @Autowired
    private IMaterialImageInfoService materialImageInfoService;
    @Autowired
    private IMaterialInfoService materialInfoService;
    @Autowired
    private SysRoleMapper sysRoleMapper;
    @Autowired
    private MaterialInfoMapper materialInfoMapper;
    @Autowired
    private IProjectMemberService memberService;


    @ApiOperation(value = "截屏-分页列表查询", notes = "截屏-分页列表查询")
    @GetMapping(value = "/pageList")
    public Result<IPage<MaterialImageInfo>> pageList(MaterialImageInfo materialImageInfo,
                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                     HttpServletRequest req) {
        Result<IPage<MaterialImageInfo>> result = new Result<>();

        QueryWrapper<MaterialImageInfo> queryWrapper = new QueryWrapper<>();
        Date createTime = materialImageInfo.getCreateTime();
        materialImageInfo.setCreateTime(null);
        if (!Check.isNull(materialImageInfo.getUserId())) {
            String roleCode = sysRoleMapper.getRoleCodeByUserId(materialImageInfo.getUserId());
            if ("admin".equals(roleCode) || "operator".equals(roleCode) || "kuaishouOperationManager".equals(roleCode) || "operationAssistant".equals(roleCode) || "touTiaoOperationManager".equals(roleCode) || "sale".equals(roleCode) || "saleDirector".equals(roleCode)) {
                materialImageInfo.setUserId(null);
                queryWrapper = QueryGenerator.initQueryWrapper(materialImageInfo, req.getParameterMap());
            } else if ("designTeamLeader".equals(roleCode) || "planeLeader".equals(roleCode) || "plane".equals(roleCode) || "plan".equals(roleCode) || "shot".equals(roleCode)) {
                String userId = materialImageInfo.getUserId();
                materialImageInfo.setUserId(null);
                queryWrapper = QueryGenerator.initQueryWrapper(materialImageInfo, req.getParameterMap());
                if (Check.isNull(materialImageInfo.getProjectId())) {
                    List<Long> projects = memberService.selectProjectsByUserId(userId);
                    if (!Check.isNull(projects)) {
                        queryWrapper.in("project_id", projects);
                    } else {
                        result.setSuccess(true);
                        result.setResult(null);
                        return result;
                    }
                }
            } else {
                queryWrapper = QueryGenerator.initQueryWrapper(materialImageInfo, req.getParameterMap());
            }
            if (!Check.isNull(createTime)) {
                try {
                    String dateString = DateUtils.getDateString(String.valueOf(createTime));
                    Map<String, Object> map = DateUtils.getStartEndTime(dateString);
                    queryWrapper.ge("create_time", map.get("start"));
                    queryWrapper.lt("create_time", map.get("end"));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
        }
        Page<MaterialImageInfo> page = new Page<>(pageNo, pageSize);
        IPage<MaterialImageInfo> pageList = materialImageInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;

    }


    /**
     * 分页列表查询
     * @param req
     * @return
     */
    @AutoLog(value = "素材-分页列表查询")
    @ApiOperation(value = "素材-分页列表查询", notes = "素材-分页列表查询")
    @GetMapping(value = "/list")
    public Result<List<MaterialImageInfo>> queryPageList(MaterialImageInfo materialImageInfo,
                                                         HttpServletRequest req) {
        Result<List<MaterialImageInfo>> result = new Result<>();
        QueryWrapper<MaterialImageInfo> queryWrapper = QueryGenerator.initQueryWrapper(materialImageInfo, req.getParameterMap());
        List<MaterialImageInfo> list = materialImageInfoService.list(queryWrapper);
        result.setSuccess(true);
        result.setResult(list);
        return result;
    }


    @GetMapping(value = "/getDetail")
    public Result<MaterialImageInfo> getDetail(String id) {
        Result<MaterialImageInfo> result = new Result<>();
        MaterialImageInfo materialImageInfo = materialImageInfoService.getById(id);
        if (!Check.isNull(materialImageInfo)) {
            MaterialInfo materialmInfo = materialInfoService.getById(materialImageInfo.getVideoId());
            if (!Check.isNull(materialmInfo)) {
                materialImageInfo.setVideoUrl(materialmInfo.getUrl());

            }

            String auditorName = materialInfoMapper.selectUserNameById(materialImageInfo.getAuditorId());
            materialImageInfo.setAuditorName(auditorName);

        }

        result.setSuccess(true);
        result.setResult(materialImageInfo);
        return result;
    }


    /**
     * 添加
     *
     * @param materialImageInfo
     * @return
     */
    @AutoLog(value = "素材-添加")
    @ApiOperation(value = "素材-添加", notes = "素材-添加")
    @PostMapping(value = "/add")
    public Result<MaterialImageInfo> add(@RequestBody MaterialImageInfo materialImageInfo) {
        Result<MaterialImageInfo> result = new Result<>();
        try {
            materialImageInfoService.save(materialImageInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param
     * @return
     */
    @AutoLog(value = "素材-编辑")

    @ApiOperation(value = "素材-编辑", notes = "素材-编辑")
    @PutMapping(value = "/edit")
    public Result<MaterialImageInfo> edit(@RequestBody MaterialImageInfo materialImageInfo) {
        Result<MaterialImageInfo> result = new Result<MaterialImageInfo>();
        MaterialImageInfo materialImageInfoEntity = materialImageInfoService.getById(materialImageInfo.getId());
        if (materialImageInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            String url = (String) materialImageInfo.getUrl();
            String urlStr = "https:" + url;
            materialImageInfo.setUrl(urlStr);
            boolean ok = materialImageInfoService.updateById(materialImageInfoEntity);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }


    @AutoLog(value = "素材-编辑")
    @ApiOperation(value = "素材-编辑", notes = "素材-编辑")
    @PutMapping(value = "/batchEdit")
    public Result<MaterialImageInfo> batchEdit(@RequestBody JSONArray updateArr) {
        Result<MaterialImageInfo> result = new Result<>();
        try {
            if (!Check.isNull(updateArr)) {
                LoginUser user = (LoginUser) SecurityUtils.getSubject()
                        .getPrincipal();
                for (int i = 0; i < updateArr.size(); i++) {
                    JSONObject updateJson = updateArr.getJSONObject(i);
                    if (!Check.isNull(updateJson)) {
                        MaterialImageInfo materialImageInfo = new MaterialImageInfo();
                        Long id = updateJson.getLong("id");
                        materialImageInfo.setId(id);
                        Integer status = updateJson.getInteger("status");
                        materialImageInfo.setStatus(status);
                        if (status == 2) {
                            materialImageInfo.setRefuseReason(updateJson.getString("refuseReason"));
                        }
                        materialImageInfo.setAuditorId(user.getId());
                        materialImageInfoService.updateById(materialImageInfo);
                    }
                }
            }
            result.success("修改成功!");
            result.setSuccess(true);
        } catch (Exception e) {
            result.setSuccess(false);
            e.printStackTrace();
        }

        return result;
    }


    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材-通过id删除")
    @ApiOperation(value = "素材-通过id删除", notes = "素材-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            materialImageInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "素材-批量删除")
    @ApiOperation(value = "素材-批量删除", notes = "素材-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<MaterialImageInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<MaterialImageInfo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            materialImageInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材-通过id查询")
    @ApiOperation(value = "素材-通过id查询", notes = "素材-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<MaterialImageInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<MaterialImageInfo> result = new Result<>();
        MaterialImageInfo materialImageInfo = materialImageInfoService.getById(id);
        if (materialImageInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(materialImageInfo);
            result.setSuccess(true);
        }
        return result;
    }
}
